/*
  Model-View-Controller demo program.
  Copyright (C) 2006 by Maria Litvin and Gary Litvin.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details:
    http://www.gnu.org/licenses/gpl.html.
*/

import java.awt.Graphics;
import java.awt.Color;
import javax.swing.JPanel;
import java.util.Observer;
import java.util.Observable;

// Represents a stretch of the road with a car picture on it.
public class Road extends JPanel implements Observer
{
  private Car car;

  // Called when the model (a Car) notifies its observers that
  // it has moved.
  public void update(Observable model, Object arg)
  {
    setBackground(Color.WHITE);
    car = (Car)model;
    repaint();
  }

  // Repaints this panel.
  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);

    // Draw the road:
    int w = getWidth();
    int h = getHeight();
    g.setColor(Color.GRAY);
    g.fillRect(0,10, w, h - 20);
    g.setColor(Color.WHITE);
    g.fillRect(0, 20 - 1, w, 2);
    g.fillRect(0, h - 20 - 1, w, 2);
    int x;
    for (x = 0; x < w; x += w/12)
    {
      g.fillRect(x, h/2 - 1, w/18, 2);
    }

    // Draw the car:
    x = 5;
    if (car != null)
      x += (int)Math.round(car.getPosition() * 0.01 * w); // w = 500 meters
    x %= w;    // wrap around
    int y = 2*h/3 - 10;

    g.translate(x, y);
    drawCar(g);
    g.translate(-x, -y);
  }

  // red
  private static int[] x01 = { 0, 1, 3,11,47,51,53,55};
  private static int[] y01 = {12, 4, 2, 1, 1, 3, 5,12};
  private static int[] z01 = {12,20,22,23,23,21,19,12};
  
  // white
  private static int[] x02 = {34,28,22,23};
  private static int[] y02 = {3,4,4,2};
  private static int[] z02 = {21,20,20,22};
  
  // white
  private static int[] x03 = {20,20,14,15};
  private static int[] y03 = {2,4,3,2};
  private static int[] z03 = {22,20,21,22};
  
  // white
  private static int[] x04 = {16,16,11,11};
  private static int[] y04 = {6,10,10,6};
  private static int[] z04 = {18,14,14,18};
  
  // white
  private static int[] x05 = {35,34,31,31};
  private static int[] y05 = {5,10,10,6};
  private static int[] z05 = {19,14,14,18};
  
  // white
  private static int[] x06 = {39,37,50,35,43,51,53,55};
  private static int[] y06 = {12, 4, 4, 2, 2, 3, 5,12};
  private static int[] z06 = {12,20,20,22,22,21,19,12};
  
  // gray
  private static int[] x07 = {50,52,52,50,49};
  private static int[] y07 = { 4, 6, 8, 7, 5};
  private static int[] z07 = {20,18,16,17,19};
  
  // red
  private static int[] x08 = {41,41,40,40};
  private static int[] y08 = {9,15,15,9};
  private static int[] z08 = {0,0,0,0};
  
  // red
  private static int[] x09 = {48,48,46,46};
  private static int[] y09 = {11,13,13,11};
  
  // red
  private static int[] x10 = {50,50};
  private static int[] y10 = {10,14};
  
  // white
  private static int[] x11 = {18,18,20,22,19,17,19};
  private static int[] y11 = {10,9,7,7,17,17,10};
  
  // white
  private static int[] x12 = {23,23,24,28,29,28,28,24,23,27,27,21,23,26,27,27,25,25};
  private static int[] y12 = {10,9,7,7,9,13,13,13,16,16,17,17,12,12,10,9,9,10};

  // white
  private static int[] x13 = {5,5};
  private static int[] y13 = {7,17};
  
  // Draws the picture of a car.
  public void drawCar(Graphics g)
  {
    g.setColor(Color.RED);
    g.fillPolygon(x01, y01, x01.length);
    g.fillPolygon(x01, z01, x01.length);
    g.setColor(Color.WHITE);
    g.fillPolygon(x02, y02, x02.length);
    g.fillPolygon(x02, z02, x02.length);
    g.fillPolygon(x03, y03, x03.length);
    g.fillPolygon(x03, z03, x03.length);
    g.fillPolygon(x04, y04, x04.length);
    g.fillPolygon(x04, z04, x04.length);
    g.fillPolygon(x05, y05, x05.length);
    g.fillPolygon(x05, z05, x05.length);
    g.fillPolygon(x06, y06, x06.length);
    g.fillPolygon(x06, z06, x06.length);
    g.setColor(Color.GRAY);
    g.fillPolygon(x07, y07, x07.length);
    g.fillPolygon(x07, z07, x07.length);
    g.setColor(Color.RED);
    g.fillPolygon(x08, y08, x08.length);
    g.fillPolygon(x09, y09, x09.length);
    g.drawLine(x10[0], y10[0], x10[1], y10[1]);
    g.setColor(Color.WHITE);
    g.fillPolygon(x11, y11, x11.length);
    g.fillPolygon(x12, y12, x12.length);
    g.drawLine(x13[0], y13[0], x13[1], y13[1]);
  }
}
